#ifndef AST_H_1027348208
#define AST_H_1027348208

#ifndef AG_OBJECT_WRAPPER_DEFINED
#define AG_OBJECT_WRAPPER_DEFINED

#ifndef AG_PLACEMENT_DELETE_REQUIRED
#if _MSC_VER >= 1200 || __INTEL_COMPILER
#define AG_PLACEMENT_DELETE_REQUIRED 1
#endif
#endif

template <class Object>
class AgObjectWrapper {
  Object object;
public:
  void operator delete(void *) {}
  void *operator new(size_t, void *p) { return p;}
#if AG_PLACEMENT_DELETE_REQUIRED
  void operator delete(void *, void *) { }
#endif
  AgObjectWrapper(const Object &o) : object(o) {}
  ~AgObjectWrapper() {}
  operator Object &() {return object;}
};

#endif

typedef union {
  double alignment;
  char ag_vt_2[sizeof(int)];
  char ag_vt_3[sizeof(long)];
  char ag_vt_4[sizeof(AgObjectWrapper<AgString >)];
  char ag_vt_5[sizeof(AgObjectWrapper<Value >)];
  char ag_vt_6[sizeof(AgObjectWrapper<AgStack<AstNode *> >)];
  char ag_vt_7[sizeof(AstNode *)];
  char ag_vt_8[sizeof(Opcode)];
  char ag_vt_9[sizeof(double)];
} parse_vs_type;

typedef enum {
  parse_white_space_token = 1, parse_script_token = 7,
  parse_statement_list_token, parse_eof_token, parse_statement_token,
  parse_open_statement_token, parse_closed_statement_token,
  parse_if_condition_token, parse_WHILE_token = 15,
  parse_expression_token = 17, parse_FOR_token = 19,
  parse_optional_expression_token, parse_simple_statement_token = 22,
  parse_DO_token, parse_compound_statement_token = 25,
  parse_dump_statement_token = 29, parse_print_statement_token,
  parse_assignment_expression_token = 42,
  parse_conditional_expression_token, parse_lvalue_token,
  parse_assignment_op_token, parse_logical_or_expression_token,
  parse_logical_and_expression_token = 49,
  parse_inclusive_or_expression_token = 51,
  parse_exclusive_or_expression_token = 53, parse_and_expression_token = 55,
  parse_equality_expression_token = 57,
  parse_relational_expression_token = 59, parse_equality_op_token,
  parse_shift_expression_token, parse_relational_op_token,
  parse_additive_expression_token, parse_shift_op_token,
  parse_multiplicative_expression_token, parse_additive_op_token,
  parse_unary_expression_token, parse_multiplicative_op_token,
  parse_factor_token, parse_unary_op_token = 71, parse_primary_token,
  parse_constant_token = 74, parse_function_call_token = 77,
  parse_optional_arg_list_token = 80, parse_arg_list_token,
  parse_string_token, parse_space_token = 109, parse_letter_token = 120,
  parse_simple_real_token = 122, parse_signed_exponent_token = 124,
  parse_integer_part_token, parse_fraction_part_token = 127,
  parse_decimal_integer_token, parse_hybrid_integer_token,
  parse_octal_integer_token, parse_exponent_token = 132, parse_digit_token,
  parse_hex_integer_token, parse_hex_digit_token = 143,
  parse_s_char_sequence_token = 146, parse_s_char_token,
  parse_escape_sequence_token = 149, parse_simple_escape_sequence_token,
  parse_octal_escape_sequence_token,
  parse_hexadecimal_escape_sequence_token, parse_one_octal_token = 164,
  parse_two_octal_token, parse_three_octal_token,
  parse_hexadecimal_digit_token = 169, parse_c_char_token = 171,
  parse_name_token = 190, parse_integer_token = 213, parse_real_token,
  parse_character_constant_token, parse_string_element_token = 242
} parse_token_type;

typedef struct parse_pcb_struct{
  parse_token_type token_number, reduction_token, error_frame_token;
  int input_code;
  int input_value;
  int line, column;
  int ssx, sn, error_frame_ssx;
  int drt, dssx, dsn;
  int ss[128];
  parse_vs_type vs[128];
  int ag_ap;
  char *error_message;
  char read_flag;
  char exit_flag;
  FileLocation input_context;
  FileLocation cs[128];
  int bts[128], btsx;
  const unsigned char * pointer;
  const unsigned char * la_ptr;
  const unsigned char *key_sp;
  int save_index, key_state;
  char ag_msg[82];
/*  Line 119, W:/agdev/agex/packages/xidek/cll/array/ast.syn */
    // Declare an abstract syntax tree
    Tree ast;

    int loopDepth;
    // Constructor for pcb
    parse_pcb_struct(const char *text);

    // Functions used during parsing
    void checkLoop();
    void reportError();
  } parse_pcb_type;

#ifndef PRULE_CONTEXT
#define PRULE_CONTEXT(pcb)  (&((pcb).cs[(pcb).ssx]))
#define PERROR_CONTEXT(pcb) ((pcb).cs[(pcb).error_frame_ssx])
#define PCONTEXT(pcb)       ((pcb).cs[(pcb).ssx])
#endif

#ifndef AG_RUNNING_CODE_CODE
/* PCB.exit_flag values */
#define AG_RUNNING_CODE         0
#define AG_SUCCESS_CODE         1
#define AG_SYNTAX_ERROR_CODE    2
#define AG_REDUCTION_ERROR_CODE 3
#define AG_STACK_ERROR_CODE     4
#define AG_SEMANTIC_ERROR_CODE  5
#endif
void init_parse(parse_pcb_type *);
void parse(parse_pcb_type *);
#endif

